#ifndef AUDIODEVICE_H_INCLUDED_C56F95A5
#define AUDIODEVICE_H_INCLUDED_C56F95A5

#include <AudioDevices/DeviceCaps.h>

//##ModelId=3A90295800E4

// This class allows one to get information about and control a NAL audio device.
// An audio device can consist of several lower level abstractions depending on
// the OS's implementation. The intent is to create uniform interface so that 
// the user does not care if, for example, the volume is implemented through  
// a mixer device or just a setting on the audio/wave device itself.
class AudioDevice
{
    public:

    typedef short sample; // 16 bit signed

    static const int SUCCESS;
    static const int FAILURE;
    static const int MAX_VOLUME;
  
    // The default block size will be used to set up buffer size, if the device
    // can not figure out how to do this on its own. Read/Write buffer sizes can
    // be changed when the device is opened.
    static const int DEFAULT_BLOCK_SIZE;

    // The caller of this function is expected to use the DeviceCaps class to
    // find the capabilities of the low level device and then choose how to
    // initialize that device.
    static DeviceCaps* getDevCaps();

    // A DevCap class should describe how the user wants the device to be
    // intialized. The user should call getDevCaps if they want to know what
    // the device supports.
    AudioDevice() {};
    ~AudioDevice() {};

    // The funciton getDevCaps(DeviceCaps *devCaps) on the low level driver 
    // should be called to figure out what the device can support before trying
    // to open the device. If the device does not support the requsted audioFormat,
    // the device will not open correctly.
    virtual int open(DevHandle inHandle, DevHandle outHandle);
    virtual bool isOpen();
    virtual void close();
  
    // buffSize is in bytes and the number of bytes read is returned.
    virtual int read(sample *appBuff, int buffSize);
    // buffSize is in bytes and the number of bytes read is returned.
    virtual int write(sample *appBuff, int buffSize);

    // Volume can be 0-100.
    virtual void setInputVol(int vol);
    virtual int getInputVol();
    virtual void setOutputVol(int vol);
    virtual int getOutputVol();

     // Returns the present sample rate and can be called any time after the
    // device has been opened.
    virtual AudioFormat::SampleRate getSampleRate();

    // Returns the present number of channels and can be called any time after
    // the device has been opened.
    virtual AudioFormat::Channel getNumChannels();

    protected:

    bool deviceOpen;

    private:
};

#endif /* AUDIODEVICE_H_INCLUDED_C56F95A5 */
