#ifndef DEVICECAPS_H_INCLUDED
#define DEVICECAPS_H_INCLUDED

#include <DevCap.h>

using namespace std;

// This class is used as a device information storage class. It is handed to 
// the low level device and that low level device fills out information in
// this class and hands it back to the caller. The caller then chooses what
// capabilities it wants to initialize the device with using this class. The
// basic unit of storage for this class is a DevCap.
class DeviceCaps : public AudioFormats
{
    public:

    DeviceCaps() {};
    ~DeviceCaps() {};
 
    int getSize();
    DevCap* getDevCap(DevHandle devHandle);

    // All get functions return NULL if what you wanted to get is not there.
    // The InDevCaps are stored in the same data structure as the OutDevCaps,
    // so they each need their own unique DevHandle;

    void setDevName(DevHandle devHandle, char* name);
    string* getDevName(DevHandle devHandle);

    void setAudioFormats(DevHandle devHandle, AudioFormats *audioFormats);
    AudioFormats* getAudioFormats(DevHandle devHandle);

    // The read block size of the low level device. This can be independent of
    // the write block size.
    void setBlockSize(DevHandle devHandle, int numBytes);
    int getBlockSize(DevHandle devHandle);

    void setInVolSupp(DevHandle devHandle, bool supported);
    bool getInVolSupp(DevHandle devHandle);

    void setOutVolSupp(DevHandle devHandle, bool supported);
    bool getOutVolSupp(DevHandle devHandle);

    // InDevCaps are used for in devices and OutDevCaps are used for out
    // devices. It is assumed that all devices are full duplex and the device
    // will try to simulate full duplex with half duplex devices if that is all
    // that is supported.
    void getDevType(DevHandle devHandle);

    protected:


    private:

    typedef map<DevHandle,DevCap> DevCapsMap;
    void checkHandle(DevHandle devHandle);
    DevCapsMap::iterator it; 
    DevCapsMap devCaps;

};

#endif
