#ifndef OSSAUDIODEVICE_H_INCLUDED_C56FE27F
#define OSSAUDIODEVICE_H_INCLUDED_C56FE27F

//##ModelId=3A902DB60114

#include <string>
#include <Common/AudioFormat.h>
#include <AudioDevices/DeviceCaps.h>
#include <AudioDevices/AudioDevice.h>

using namespace std;

// This can be used to loop through the sampleRates.
static AudioFormat::SampleRate sampleRates[NUM_SAMPLE_RATES] = {
    AudioFormat::S8K,
    AudioFormat::S11K,
    AudioFormat::S16K,
    AudioFormat::S22K,
    AudioFormat::S24K,
    AudioFormat::S32K,
    AudioFormat::S44K,
    AudioFormat::S48K
};

// This can be used to loop through the encodings.
static AudioFormat::Encoding encodings[NUM_ENCODINGS] = {
    AudioFormat::PCM,
    AudioFormat::PCMU,
    AudioFormat::PCMA,
    AudioFormat::S8,
    AudioFormat::U8,
    AudioFormat::S16
};

class OSSAudioDevice : public AudioDevice
{
    public:

    static const int MAX_NUM_DEVS;
    static const int MAX_NUM_MIXERS;
    static const int MAX_OSS_NAME_SIZE;
    static const int MAX_NUM_OSS_DEVS;

    // Since only the lower level device can correctly fill out a device
    // capability class, this function is not inherited from AudioDevice. 
    // The caller of this function is expected to use the DeviceCaps class to
    // find the capabilities of the low level device and then choose how to
    // initialize that device.
    static DeviceCaps* getDevCaps();

    // A DevCap class should describe how the user wants the device to be
    // intialized. The user should call getDevCaps if they want to know what the
    // device supports.
    OSSAudioDevice(){};
    ~OSSAudioDevice(){};

    int open();
    bool isOpen();
    void close();

    // buffSize is in bytes and the number of bytes read is returned.
    int read(sample *appBuff, int buffSize);
    // buffSize is in bytes and the number of bytes read is returned.
    int write(sample *appBuff, int buffSize);
   
    void setInputVol(int vol);
    int getInputVol();
    void setOutputVol(int vol);
    int getOutputVol();

    AudioFormat::SampleRate getSampleRate();
    AudioFormat::Channel getNumChannels();

    protected:

    private:

    static int NALEncoding2OSS(AudioFormat::Encoding encoding);
    // It may be necessary to do some convertion of volumes so the lower level
    // device understands the 0-100 volume scale of the NAL.
    static int vol2Dev(int vol);

    // audio device    
    static const string AUDIO_STR;
    string audioName; // the name the OSS gives the device
    string audioDevName; // the file system /dev name
    int audioDesc; // the same descriptor is used for mixer ioctl's
    int readBuffSize; // in bytes
    int writeBuffSize; // in bytes
    int writeBlockSize; // in bytes
    int readBlockSize; // in bytes

    // mixer device
    static const string MIXER_STR;
    string mixerName; // the name the OSS gives the mixer
    string mixerDevName; // the file system /dev name
    int readDevMask;
    int readRecMask;

    bool getAudioCaps(int audioNum, DeviceCaps *pDeviceCaps);
    bool getMixerCaps(int mixerNum, DeviceCaps *pDeviceCaps);

};

#endif /* OSSAUDIODEVICE_H_INCLUDED_C56FE27F */
