#ifdef WIN32
#pragma warning (disable:4786)
#endif

#include <assert.h>
#include <AudioDevices/Win32AudioDevice.h>

const int Win32AudioDevice::MAX_WAVE_VOLUME = 0xffff;
const int Win32AudioDevice::NUM_BLOCKS = 128;
const int Win32AudioDevice::DEFAULT_BLOCK_SIZE = 512;

Win32AudioDevice::Win32AudioDevice()
{
    // AUDIT register function
    readBlockSize = DEFAULT_BLOCK_SIZE;
    writeBlockSize = DEFAULT_BLOCK_SIZE;
    mixerClosed = true;
    waveInClosed = true;
    waveOutClosed = true;
    readThread = CreateThread(NULL,1024,
			     (LPTHREAD_START_ROUTINE)Win32AudioDevice::readProc,
			      this,0,&readThreadID);
}

Win32AudioDevice::~Win32AudioDevice()
{
    // AUDIT register function
	
    if(deviceOpen)
    {
	this->close();
    }
    else
    {
	// AUDIT device open on destruction
    }
}

DeviceCaps*
Win32AudioDevice::getDevCaps()
{
    Win32DeviceCaps *deviceCaps = new Win32DeviceCaps();
    
    numInWaves = waveInGetNumDevs();
    numOutWaves = waveOutGetNumDevs();

    int dev;
    for(dev = 0;dev < numInWaves;dev++)
    {
	if(getWaveInCaps(dev, deviceCaps) != true)
	{
	    // AUDIT Could not get waveIn audio device capabilities dev.
	}
    }

    for(;dev < numInWaves+numOutWaves;dev++)
    {
	if(getWaveOutCaps(dev, deviceCaps) != true)
	{
	    // AUDIT Could not get waveOut audio device capabilities dev.
	}
    }

    // should this be copied?????
    win32DeviceCaps = deviceCaps;
    return deviceCaps;
}

bool
Win32AudioDevice::getWaveInCaps(int audioDevNum, Win32DeviceCaps *deviceCaps)
{
    WAVEINCAPS waveInCaps;
    MMRESULT mmresult = waveInGetDevCaps(audioDevNum,&waveInCaps,sizeof(WAVEINCAPS));
    if(mmresult != MMSYSERR_NOERROR)
    {
	// AUDIT Problem getting waveInCaps.
        return false;
    }

    // This seems to be the only way to relate a mixer with the waveIn device.
    MIXERLINE mixerLine;
    mixerLine.cbStruct = sizeof(mixerLine);
    mixerLine.Target.dwType = MIXERLINE_TARGETTYPE_WAVEIN;
    strncpy(mixerLine.Target.szPname, waveInCaps.szPname, MAXPNAMELEN);
    mixerLine.Target.vDriverVersion = waveInCaps.vDriverVersion;
    mixerLine.Target.wMid = waveInCaps.wMid;
    mixerLine.Target.wPid = waveInCaps.wPid;

    mmresult = mixerGetLineInfo((HMIXEROBJ)audioDevNum, &mixerLine,
			        MIXER_OBJECTF_MIXER | MIXER_GETLINEINFOF_TARGETTYPE);

    if(mmresult == MMSYSERR_NOERROR)
    {
	deviceCaps->setRecLine(audioDevNum, mixerLine.dwLineID);
	deviceCaps->setInVolSupp(audioDevNum, true);
    }
    else
    {
	// AUDIT Problem getting mixerLineInfo.
        return false;
    }

    deviceCaps->setDevName(audioDevNum, waveInCaps.szPname);
    // AUDIT print waveInCaps.szPname

    // Check which sampling modes are supported.
    AudioFormats *audioFormats = new AudioFormats();
    for(int rate = 0; rate < NUM_SAMPLE_RATES;rate++)
    {
	if(NAL2WaveFormat(waveInCaps.dwFormats,sampleRates[rate],
                      AudioFormat::MONO,AudioFormat::B8))
	{
	   audioFormats->setSampleRate(rate,sampleRates[rate]);
	   setInEncodings(audioDevNum,audioFormats,sampleRates[rate],
                      AudioFormat::MONO,AudioFormat::B8);
	   audioFormats->setChannel(rate,AudioFormat::MONO);
	}
	else if(NAL2WaveFormat(waveInCaps.dwFormats,sampleRates[rate],
                           AudioFormat::MONO,AudioFormat::B16))
	{
	   audioFormats->setSampleRate(rate,sampleRates[rate]);
	   setInEncodings(audioDevNum,audioFormats,sampleRates[rate],
                      AudioFormat::MONO,AudioFormat::B16);
	   audioFormats->setChannel(rate,AudioFormat::MONO);
	}
	else if(NAL2WaveFormat(waveInCaps.dwFormats,sampleRates[rate],
            AudioFormat::STEREO,AudioFormat::B8))
	{
	   audioFormats->setSampleRate(rate,sampleRates[rate]);
	   setInEncodings(audioDevNum,audioFormats,sampleRates[rate],
                      AudioFormat::STEREO,AudioFormat::B8);
	   audioFormats->setChannel(rate,AudioFormat::STEREO);
	}
	else if(NAL2WaveFormat(waveInCaps.dwFormats,sampleRates[rate],
                           AudioFormat::STEREO,AudioFormat::B16))
	{
	   audioFormats->setSampleRate(rate,sampleRates[rate]);
	   setInEncodings(audioDevNum, audioFormats,sampleRates[rate],
                      AudioFormat::STEREO,AudioFormat::B16);
	   audioFormats->setChannel(rate,AudioFormat::STEREO);
	}
    }

    deviceCaps->setAudioFormats(audioDevNum, audioFormats);
    // Store the desciptor for later use. What should the second arg be??
    deviceCaps->setWaveInDesc(audioDevNum, audioDevNum);

    return true;
}

bool
Win32AudioDevice::getWaveOutCaps(int audioDevNum, Win32DeviceCaps *deviceCaps)
{
    WAVEOUTCAPS waveOutCaps;
    MMRESULT mmresult = waveOutGetDevCaps(audioDevNum,&waveOutCaps,sizeof(WAVEOUTCAPS));
    if(mmresult != MMSYSERR_NOERROR)
    {
	// AUDIT Problem getting waveOutCaps.
        return false;
    }

    deviceCaps->setDevName(audioDevNum, waveOutCaps.szPname);
    // AUDIT print waveOutCaps.szPname

    // Check which sampling modes are supported.
    AudioFormats *audioFormats = new AudioFormats();
    for(int rate = 0; rate < NUM_SAMPLE_RATES;rate++)
    {
	if(NAL2WaveFormat(waveOutCaps.dwFormats,sampleRates[rate],
       AudioFormat::MONO,AudioFormat::B8))
	{
	   audioFormats->setSampleRate(rate,sampleRates[rate]);
	   setOutEncodings(audioDevNum,audioFormats,sampleRates[rate],
                       AudioFormat::MONO,AudioFormat::B8);
	   audioFormats->setChannel(rate,AudioFormat::MONO);
	}
	else if(NAL2WaveFormat(waveOutCaps.dwFormats,sampleRates[rate],
            AudioFormat::MONO,AudioFormat::B16))
	{
	   audioFormats->setSampleRate(rate,sampleRates[rate]);
	   setOutEncodings(audioDevNum,audioFormats,sampleRates[rate],
                       AudioFormat::MONO,AudioFormat::B16);
	   audioFormats->setChannel(rate,AudioFormat::MONO);
	}
	else if(NAL2WaveFormat(waveOutCaps.dwFormats,sampleRates[rate],
            AudioFormat::STEREO,AudioFormat::B8))
	{
	   audioFormats->setSampleRate(rate,sampleRates[rate]);
	   setOutEncodings(audioDevNum,audioFormats,sampleRates[rate],
                       AudioFormat::STEREO,AudioFormat::B8);
	   audioFormats->setChannel(rate,AudioFormat::STEREO);
	}
	else if(NAL2WaveFormat(waveOutCaps.dwFormats,sampleRates[rate],
            AudioFormat::STEREO,AudioFormat::B16))
	{
	   audioFormats->setSampleRate(rate,sampleRates[rate]);
	   setOutEncodings(audioDevNum,audioFormats,sampleRates[rate],
                       AudioFormat::STEREO,AudioFormat::B16);
	   audioFormats->setChannel(rate,AudioFormat::STEREO);
	}
    }

    if(waveOutCaps.dwSupport & WAVECAPS_VOLUME)
    {
	deviceCaps->setOutVolSupp(audioDevNum,true);
    }

    deviceCaps->setAudioFormats(audioDevNum, audioFormats);
    // Store the desciptor for later use. What should the second arg be???
    deviceCaps->setWaveOutDesc(audioDevNum, audioDevNum);

    return true;
}

void
Win32AudioDevice::setInEncodings(int waveIn, AudioFormats *audioFormats,
                                 AudioFormat::SampleRate rate, 
                                 AudioFormat::Channel channels,
		                         AudioFormat::NumBits bits)
{
    WAVEFORMATEX waveTex;

    // Check which audio encodings are supported.
    for(int encoding = 0;encoding < NUM_ENCODINGS;encoding++)
    {
        waveTex.wFormatTag      = NAL2WaveEncoding(encodings[encoding]);
        waveTex.nChannels       = channels;
        waveTex.nSamplesPerSec  = rate;
        waveTex.nAvgBytesPerSec = waveTex.nSamplesPerSec * waveTex.nBlockAlign;
        waveTex.nBlockAlign     = waveTex.wBitsPerSample / 8 * waveTex.nChannels;
        waveTex.wBitsPerSample  = bits;
        waveTex.cbSize = 0;

        MMRESULT mmresult = waveInOpen(NULL, (UINT)waveIn, &waveTex, (UINT)NULL, (UINT)NULL, WAVE_FORMAT_QUERY);
        
        if (mmresult != MMSYSERR_NOERROR)
	    {
	    // AUDIT could not query wave in device.
        }


	// How do yo get the encoding??
	audioFormats->setEncoding(waveIn,(AudioFormat::Encoding)sampleRates[encoding]);
    }
}

void
Win32AudioDevice::setOutEncodings(int waveOut, AudioFormats *audioFormats,
                                  AudioFormat::SampleRate rate, 
                                  AudioFormat::Channel channel,
                                  AudioFormat::NumBits bits)
{
    WAVEFORMATEX waveTex;

    // Check which audio encodings are supported.
    for(int encoding = 0;encoding < NUM_ENCODINGS;encoding++)
    {
        waveTex.wFormatTag      = NAL2WaveEncoding(encodings[encoding]);
        waveTex.nChannels       = channel;
        waveTex.nSamplesPerSec  = rate;
        waveTex.nAvgBytesPerSec = waveTex.nSamplesPerSec * waveTex.nBlockAlign;
        waveTex.nBlockAlign     = waveTex.wBitsPerSample / 8 * waveTex.nChannels;
        waveTex.wBitsPerSample  = bits;
        waveTex.cbSize = 0;
        
        MMRESULT mmresult = waveOutOpen(NULL, (UINT)waveOut, &waveTex, (UINT)NULL, (UINT)NULL, WAVE_FORMAT_QUERY);
            if (mmresult != MMSYSERR_NOERROR)
	{
	    // AUDIT could not query wave in device.
        }

	// How do yo get the encoding??
	audioFormats->setEncoding(waveOut,(AudioFormat::Encoding)sampleRates[encoding]);
    }
}

int
Win32AudioDevice::NAL2WaveEncoding(AudioFormat::Encoding encoding)
{
    switch(encoding)
    {	
    case AudioFormat::PCM:  return WAVE_FORMAT_PCM;
	case AudioFormat::PCMU: return WAVE_FORMAT_MULAW;
	case AudioFormat::PCMA: return WAVE_FORMAT_ALAW;
	case AudioFormat::S8:   return WAVE_FORMAT_UNKNOWN;
	case AudioFormat::U8:   return WAVE_FORMAT_UNKNOWN;
	case AudioFormat::S16:  return WAVE_FORMAT_UNKNOWN;
    };

    return AudioFormat::NO_FORMAT;
}

bool
Win32AudioDevice::NAL2WaveFormat(DWORD dwFormat, AudioFormat::SampleRate rate, 
                                 AudioFormat::Channel channel, 
                                 AudioFormat::NumBits bits)
{
    if(channel == AudioFormat::MONO)
    {
	if(rate == AudioFormat::S11K)
	{
	    if(bits == AudioFormat::B8)
	    {
		// WAVE_FORMAT_1M08 11.025 kHz, Mono, 8-bit
		if(WAVE_FORMAT_1M08 & dwFormat)
		{
		    return true;
		}
	    }
	    else if(bits == AudioFormat::B16)
	    {
		// WAVE_FORMAT_1M16 11.025 kHz, Mono, 16-bit
		if(WAVE_FORMAT_1M16 & dwFormat)
		{
		    return true;
		}
	    }
	}
	else if(rate == AudioFormat::S22K)
	{
	    if(bits == AudioFormat::B8)
	    {
		// WAVE_FORMAT_2M08 22.05 kHz, Mono, 8-bit
		if(WAVE_FORMAT_2M08 & dwFormat)
		{
		    return true;
		}
	    }
	    else if(bits == AudioFormat::B16)
	    {
		// WAVE_FORMAT_2M16 22.05 kHz, Mono, 16-bit
		if(WAVE_FORMAT_2M16 & dwFormat)
		{
		    return true;
		}
	    }
	}
	else if(rate == AudioFormat::S44K)
	{
	    if(bits == AudioFormat::B8)
	    {
		// WAVE_FORMAT_4M08 44.1 kHz, Mono, 8-bit
		if(WAVE_FORMAT_4M08 & dwFormat)
		{
		    return true;
		}
	    }
	    else if(bits == AudioFormat::B16)
	    {
		// WAVE_FORMAT_4M16 44.1 kHz, Mono, 16-bit
		if(WAVE_FORMAT_4M16 & dwFormat)
		{
		    return true;
		}
	    }
	}
    }
    else if (channel == AudioFormat::STEREO)
    {
	if(rate == AudioFormat::S11K)
	{	
	    if(bits == AudioFormat::B8)
	    {
		// WAVE_FORMAT_1S08 11.025 kHz, Stereo, 8-bit
		if(WAVE_FORMAT_1S08 & dwFormat)
		{
		    return true;
		}
	    }
	    else if(bits == AudioFormat::B16)
	    {
		// WAVE_FORMAT_1S16 11.025 kHz, Stereo, 16-bit
		if(WAVE_FORMAT_1S16 & dwFormat)
		{
		    return true;
		}
	    }
	}
	else if(rate == AudioFormat::S22K)
	{
	    if(bits == AudioFormat::B8)
	    {
		// WAVE_FORMAT_2S08 22.05 kHz, Stereo, 8-bit
		if(WAVE_FORMAT_2S08 & dwFormat)
		{
		    return true;
		}
	    }
	    else if(bits == AudioFormat::B16)
	    {
		// WAVE_FORMAT_2S16 22.05 kHz, Stereo, 16-bit
		if(WAVE_FORMAT_2S16 & dwFormat)
		{
		    return true;
		}
		return true;
	    }
	}
	else if(rate == AudioFormat::S44K)
	{
	    if(bits == AudioFormat::B8)
	    {
		// WAVE_FORMAT_4S08 44.1 kHz, Stereo, 8-bit
		if(WAVE_FORMAT_4S08 & dwFormat)
		{
		    return true;
		}
	    }
	    else if(bits == AudioFormat::B16)
	    {
		// WAVE_FORMAT_4S16 44.1 kHz, Stereo, 16-bit
		if(WAVE_FORMAT_4S16 & dwFormat)
		{
		    return true;
		}
	    }
	}
    }
    else
    {
	// WAVE_INVALIDFORMAT invalid format 
	return false;
    }

    return false;
}

// Open the following win32 multimedia devices:
//	waveIn
//	waveOut
//	mixer
int
Win32AudioDevice::open(DevHandle inHandle, DevHandle outHandle)
{
    // AUDIT register function

    openInHandle = inHandle;
    openOutHandle = outHandle;    
    if(deviceOpen)
    {
	// AUDIT The win32 device already open.
	//	 Close it before trying to open it.
	return FAILURE;
    }

    if(openWaveIn() == FAILURE)
    {
	// AUDIT Could not open the wave in device.
	return FAILURE;
    }

    if(openMixer() == FAILURE)
    {
	// AUDIT Could not open the mixer device.
	return FAILURE;
    }

    if(openWaveOut() == FAILURE)
    {
	// AUDIT Could not open the wave out device.
	return FAILURE;
    }

    return SUCCESS;
}

// Setup a buffer and open the wave in device.
// This device records sound from the microphone. 
int
Win32AudioDevice::openWaveIn()
{
    // AUDIT register function

    readBlockSize = win32DeviceCaps->getBlockSize(openInHandle);

    if(waveInHandle)
    {
	// AUDIT The wave input device is already open.
    }

    // setup buffers and release if necessary
    if(waveInHdrs != NULL)
    {
	delete waveInHdrs;
    }

    waveInHdrs = new WAVEHDR[NUM_BLOCKS];

    if(waveInBuf != NULL)
    {
	delete waveInBuf;
    }

    waveInBuf = new char[NUM_BLOCKS*readBlockSize];

    currWaveHdr = NULL;

    DWORD hmmm = NULL;

    MMRESULT mmresult;
    mmresult = waveInOpen(&waveInHandle, win32DeviceCaps->getWaveInDesc(openInHandle), waveTex,
			  (DWORD)readProc, NULL, CALLBACK_FUNCTION);

    // set the waveInHdrs to point into wavInBuf
    for (int i = 0; i < NUM_BLOCKS; i++) 
    {
	waveInHdrs[i].lpData = waveInBuf + i * readBlockSize;
	waveInHdrs[i].dwBufferLength = readBlockSize;
	waveInHdrs[i].dwFlags = 0;
	mmresult = waveInPrepareHeader(waveInHandle, &waveInHdrs[i],
				       sizeof(WAVEHDR));
	assert(mmresult == MMSYSERR_NOERROR);
	mmresult = waveInAddBuffer(waveInHandle, &waveInHdrs[i],
				   sizeof(WAVEHDR));
	assert(mmresult == MMSYSERR_NOERROR);
    }

    int err = waveInStart(waveInHandle);

    if(err)
    {
	waveOutGetErrorText(err, errorText, sizeof(errorText));
	// AUDIT Error during waveInStart "errorText".
	exit(1);
    }

    waveInClosed = false;
    return SUCCESS;
}

void
Win32AudioDevice::readProc(Win32AudioDevice *win32Dev, HWAVEIN hwaveIn, 
			   UINT uMsg, DWORD dwInstance, DWORD dwParam1,
			   DWORD dwParam2)
{
    win32Dev->waveInProc(hwaveIn, uMsg, dwInstance, dwParam1, dwParam2);
}

void
Win32AudioDevice::waveInProc(HWAVEIN hwaveIn, UINT uMsg, DWORD dwInstance,
			      DWORD dwParam1, DWORD dwParam2)
{
    // AUDIT register function

    if(uMsg != WIM_DATA)
    {
	// AUDIT Called without using WIM_DATA.
	return;
    }

    WAVEHDR *inWaveHdr;
    WAVEHDR **insertWaveHdr;

    // the wave header from the callback
    // gets its next pointer nulled
    inWaveHdr = (WAVEHDR*)dwParam1;
    inWaveHdr->lpNext = NULL;

    // start inserting where the
    // currWaveHdr left off 
    insertWaveHdr = &currWaveHdr;

    // loop until some pointer is
    // null. when would that be?
    while(*insertWaveHdr != NULL)
    {
	// dereference the next pointer and
	// get the address of that and move on
	insertWaveHdr = &((*insertWaveHdr)->lpNext);
    }
    
    // put the current waveheader's address back. why??
    *insertWaveHdr = inWaveHdr;
    SetEvent(readyHandle);
}

// Setup a buffer and open the wave out device.
// This device plays sound to the speakers. For
// now only 16 bit audio format is supported.
int
Win32AudioDevice::openWaveOut()
{
    // AUDIT register function

    writeBlockSize = win32DeviceCaps->getBlockSize(openOutHandle);

    if(waveOutHandle)
    {
	// AUDIT The wave input device is already open.
	return SUCCESS;
    }

    // setup buffers and release if necessary
    if(waveOutHdrs != NULL)
    {
	delete waveOutHdrs;
    }

    if(waveOutBuf != NULL)
    {
	delete waveOutBuf;
    }

    waveOutBuf = new char[NUM_BLOCKS*writeBlockSize];

    MMRESULT mmresult = waveOutOpen(&waveOutHandle, win32DeviceCaps->getWaveOutDesc(openOutHandle), waveTex,
			   NULL, NULL, CALLBACK_NULL);

    // get new buffer for each wave header
    for (int i = 0; i < NUM_BLOCKS; i++)
    {
	waveOutHdrs[i].dwFlags = 0;
	waveOutHdrs[i].dwBufferLength = writeBlockSize;
	waveOutHdrs[i].lpData = waveOutBuf + i * writeBlockSize;
	waveOutHdrs[i].dwUser = i;
	mmresult = waveOutPrepareHeader(waveOutHandle, &waveOutHdrs[i],
				       sizeof(WAVEHDR));
	waveOutHdrs[i].dwFlags |= WHDR_DONE;
	assert(mmresult == MMSYSERR_NOERROR);
    }

    waveOutClosed = false; 
    return SUCCESS;
}


void
Win32AudioDevice::close()
{
    // AUDIT register function

    if(this->closeWaveIn())
    {
	// AUDIT Closed wave in device.
    }
    else
    {
	// AUDIT Could not close wave in device.
    }

    if(this->closeWaveOut())
    {
	// AUDIT Closed wave out device.
    }
    else
    {
	// AUDIT Could not close wave out device.
    }

    // AUDIT Closing wave out device.

    if(this->closeMixer())
    {
	// AUDIT Closed mixer device.
    }
    else
    {
	// AUDIT Could not properly mixer device.
    }

    if(deviceOpen)
    {
	deviceOpen = FALSE;
	// AUDIT Closed win32 audio device.
    }
    else
    {
	// AUDIT Could not properly close win32 audio device.
    }
}

int
Win32AudioDevice::closeWaveIn()
{
    // AUDIT register function

    if(waveInHandle)
    {
	return AudioDevice::SUCCESS;
    }

    waveInReset(waveInHandle);

    for(int i = 0;i < NUM_BLOCKS;i++)
    {
	if(waveInHdrs[i].dwFlags & WHDR_PREPARED)
	{
	    waveInUnprepareHeader(waveInHandle,&waveInHdrs[i],
				   sizeof(WAVEHDR));
	}
	else
	{
	    // AUDIT A WAVEHDR was not prepared properly.
	}
    }

    waveInClose(waveInHandle);

    delete waveInHdrs;
    waveInHdrs = NULL;
    delete waveInBuf;
    waveInBuf = NULL;
    currWaveHdr = NULL;
    waveInClosed = true;
    openInHandle = -1;

    return AudioDevice::SUCCESS;
}

int
Win32AudioDevice::read(sample *appBuff, int appBuffSize)
{
    // AUDIT register function

    if(appBuff == NULL)
    {
	return NULL;
    }

    // Since buffSize may not be a multiple of NUK_BLOCKS, the size of the last (or only)
    // block must determined.
    int lastBlockSize = NUM_BLOCKS % appBuffSize;
    int restBuffSize = appBuffSize - lastBlockSize;

    WAVEHDR *devWaveHdr; // used to hand a waveheader to device
    MMRESULT mmresult;
    int bytesCopied = 0;

    // copy everything of size bytesPerBlock to device
    while(currWaveHdr != NULL && bytesCopied < restBuffSize)
    {
	devWaveHdr = currWaveHdr;
	// Step though the wave headers pointing into the waveInBuf and copy
	// the wave headers' data into the wave device's buffers.
	memcpy(appBuff, devWaveHdr->lpData, readBlockSize);
	currWaveHdr = currWaveHdr->lpNext;
	assert(devWaveHdr->dwFlags & WHDR_DONE);
	assert(devWaveHdr->dwFlags & ~WHDR_INQUEUE);
	devWaveHdr->lpNext          = NULL; 
	devWaveHdr->dwBytesRecorded = 0; 
	devWaveHdr->dwFlags        &= ~WHDR_DONE;
	mmresult = waveInAddBuffer(waveInHandle, devWaveHdr, sizeof(WAVEHDR)); 
	assert(mmresult == MMSYSERR_NOERROR);
	assert(devWaveHdr->dwFlags & WHDR_INQUEUE);
	bytesCopied += readBlockSize;
    }

    // copy the last block (or the only block) to device
    if(lastBlockSize != NULL)
    {
	devWaveHdr = currWaveHdr;
	memcpy(appBuff, devWaveHdr->lpData, lastBlockSize);
	currWaveHdr = currWaveHdr->lpNext;
	assert(devWaveHdr->dwFlags & WHDR_DONE);
	assert(devWaveHdr->dwFlags & ~WHDR_INQUEUE);
	devWaveHdr->lpNext          = NULL; 
	devWaveHdr->dwBytesRecorded = 0; 
	devWaveHdr->dwFlags        &= ~WHDR_DONE;
	mmresult = waveInAddBuffer(waveInHandle, devWaveHdr, sizeof(WAVEHDR)); 
	assert(mmresult == MMSYSERR_NOERROR);
	assert(devWaveHdr->dwFlags & WHDR_INQUEUE);
	bytesCopied += lastBlockSize;
    }

    return bytesCopied;
}

int
Win32AudioDevice::closeWaveOut()
{
    // AUDIT register function

    if(waveOutHandle)
    {
	return SUCCESS;
    }

    waveOutReset(waveOutHandle);

    for(int i = 0;i < NUM_BLOCKS;i++)
    {
	if(waveOutHdrs[i].dwFlags & WHDR_PREPARED)
	{
	    waveOutUnprepareHeader(waveOutHandle,&waveOutHdrs[i],
				   sizeof(WAVEHDR));
	}
	else
	{
	    // AUDIT A WAVEHDR was not prepared properly.
	}
    }

    waveOutClose(waveOutHandle);

    delete waveOutHdrs;
    waveOutHdrs = NULL;
    delete waveOutBuf;
    waveOutBuf = NULL;
    waveOutClosed = true;
    openOutHandle = -1;

    return AudioDevice::SUCCESS;
}

int
Win32AudioDevice::write(sample *appBuff, int appBuffSize)
{
    // AUDIT register function

    if(appBuff == NULL)
    {
	return NULL;
    }

    // Since buffSize may not be a multiple of NUM_BLOCKS, the size of the last (or only)
    // block must determined.
    int lastBlockSize = NUM_BLOCKS % appBuffSize;
    int restBuffSize = appBuffSize - lastBlockSize;

    WAVEHDR *devWaveHdr; // used to hand a waveheader to device
    MMRESULT mmresult;
    int bytesCopied = 0;
   
    if(writeBlockSize < appBuffSize ) // the buffer is larger than one block
    { 
	while(bytesCopied < restBuffSize)
	{
	    devWaveHdr = this->getOutHdr();
    
	    if(devWaveHdr == NULL)
	    {
		// AUDIT Ran out of space in win32device buffer.
		break;
	    }

	    memcpy(appBuff, devWaveHdr->lpData, writeBlockSize);
	    bytesCopied += writeBlockSize;
	    mmresult = waveOutWrite(waveOutHandle, devWaveHdr, sizeof(WAVEHDR));

	    //if(mmresult == WRITE_ERROR_STILL_PLAYING)
	    //{
		// AUDIT Wave device can not accept more data.
		//break;
	    //}
	}

	if(lastBlockSize != NULL)
	{
	    memcpy(appBuff, devWaveHdr->lpData, lastBlockSize);
	    bytesCopied += lastBlockSize;
	    mmresult = waveOutWrite(waveOutHandle, devWaveHdr, sizeof(WAVEHDR));

	    //if(mmresult == WRITE_ERROR_STILL_PLAYING)
	    //{
		// AUDIT Wave device can not accept more data.
	    //}
	}
    }
    else // the buffer is smaller than one block
    {
	devWaveHdr = this->getOutHdr();
	memcpy(appBuff, devWaveHdr->lpData, lastBlockSize);
	bytesCopied += lastBlockSize;
	mmresult = waveOutWrite(waveOutHandle, devWaveHdr, sizeof(WAVEHDR));

	//if(mmresult == WRITE_ERROR_STILL_PLAYING)
	//{
	    // AUDIT Wave device can not accept more data.
	//}
    }

    return bytesCopied;
}

WAVEHDR*
Win32AudioDevice::getOutHdr()
{
    WAVEHDR *waveHdr = NULL;

    for(int i = 0;i < NUM_BLOCKS;i++)
    {
	assert(waveOutHdrs[i].dwFlags & WHDR_PREPARED);
	if(waveOutHdrs[i].dwFlags & WHDR_DONE)
	{
	    waveOutHdrs[i].dwFlags &= WHDR_PREPARED;
	    // AUDIT Wave header number i found.
	    waveHdr = &waveOutHdrs[i];
	}
    }

    return waveHdr;
}

int
Win32AudioDevice::openMixer()
{
    if(!mixerClosed)
    {
	closeMixer();
	mixerHandle = 0;
    }

    MMRESULT mmresult = mixerOpen(&mixerHandle, win32DeviceCaps->getMixerDesc(openInHandle),
				  (unsigned long)NULL, (unsigned long)NULL, MIXER_OBJECTF_MIXER);

    if(mmresult != MMSYSERR_NOERROR)
    {
	// AUDIT Could not open mixer.
	return FAILURE;
    }

    return SUCCESS;
}

int
Win32AudioDevice::closeMixer()
{
    // AUDIT register function
    // save the mixer configuration???

    MMRESULT mmresult;
    mmresult = mixerClose(mixerHandle);
    if(mmresult != MMSYSERR_NOERROR)
    {
	// AUDIT Could not close mixer.
	return FAILURE;
    }

    mixerClosed = true;
    return SUCCESS;
}

int
Win32AudioDevice::vol2Dev(int volume)
{
    // AUDIT register function
    return ((volume * MAX_WAVE_VOLUME / MAX_VOLUME) << 16 |
	    (volume * MAX_WAVE_VOLUME / MAX_VOLUME));
}

int
Win32AudioDevice::dev2Vol(int vol)
{
    // AUDIT volume set to...
    return ((vol & 0xffff) * MAX_VOLUME / MAX_WAVE_VOLUME);
}

void
Win32AudioDevice::setInputVol(int vol)
{
    MIXERCONTROLDETAILS_UNSIGNED uMixerCtrlDetails;
    MIXERCONTROLDETAILS mixerCtrlDetails;
    MIXERLINECONTROLS mixerLineCtrls;
    MIXERCONTROL mixerCtrl;
    MMRESULT mmresult;

    mixerLineCtrls.cbStruct = sizeof(mixerLineCtrls);
    mixerLineCtrls.pamxctrl = &mixerCtrl;
    mixerLineCtrls.cbmxctrl = sizeof(MIXERCONTROL);
    mixerLineCtrls.dwLineID = win32DeviceCaps->getRecLine(openInHandle);
    mixerLineCtrls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;

    mmresult = mixerGetLineControls((HMIXEROBJ)mixerHandle, &mixerLineCtrls,
				    MIXER_GETLINECONTROLSF_ONEBYTYPE |
				    MIXER_OBJECTF_HMIXER);

    if (mmresult != MMSYSERR_NOERROR)
    {
	// AUDIT could not get mixer line controls.
	}

    mixerCtrlDetails.cbStruct = sizeof(mixerCtrlDetails);
    mixerCtrlDetails.dwControlID = mixerCtrl.dwControlID;
    mixerCtrlDetails.cChannels = 1;
    mixerCtrlDetails.cMultipleItems = mixerCtrl.cMultipleItems;
    mixerCtrlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
    mixerCtrlDetails.paDetails = &uMixerCtrlDetails;
    uMixerCtrlDetails.dwValue = ((mixerCtrl.Bounds.dwMaximum - mixerCtrl.Bounds.dwMinimum)
		     * vol)/MAX_WAVE_VOLUME;

    mmresult = mixerSetControlDetails((HMIXEROBJ)mixerHandle, &mixerCtrlDetails,
				      MIXER_OBJECTF_HMIXER);

    if (mmresult != MMSYSERR_NOERROR)
    {
	// AUDIT could not set mixer controls details.
    }
}

int
Win32AudioDevice::getInputVol()
{
    MIXERCONTROLDETAILS_UNSIGNED uMixerCtrlDetails;
    MIXERCONTROLDETAILS mixerCtrlDetails;
    MIXERLINECONTROLS mixerLineCtrl;
    MIXERCONTROL mixerCtrl;
    MMRESULT mmresult;

    mixerLineCtrl.cbStruct = sizeof(mixerLineCtrl);
    mixerLineCtrl.pamxctrl = &mixerCtrl;
    mixerLineCtrl.cbmxctrl = sizeof(MIXERCONTROL);
    mixerLineCtrl.dwLineID = win32DeviceCaps->getRecLine(openInHandle);
    mixerLineCtrl.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;

    mmresult = mixerGetLineControls((HMIXEROBJ)mixerHandle, &mixerLineCtrl,
				    MIXER_GETLINECONTROLSF_ONEBYTYPE |
				    MIXER_OBJECTF_HMIXER);

    if (mmresult != MMSYSERR_NOERROR)
    {
	return 0;        
    }

    mixerCtrlDetails.cbStruct = sizeof(mixerCtrlDetails);
    mixerCtrlDetails.dwControlID = mixerCtrl.dwControlID;
    mixerCtrlDetails.cChannels = 1;
    mixerCtrlDetails.cMultipleItems = mixerCtrl.cMultipleItems;
    mixerCtrlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
    mixerCtrlDetails.paDetails = &uMixerCtrlDetails;

    mmresult = mixerGetControlDetails((HMIXEROBJ)mixerHandle, &mixerCtrlDetails,
				      MIXER_OBJECTF_HMIXER);
    if (mmresult != MMSYSERR_NOERROR)
    {
	return 0;
    }

    return (int)(uMixerCtrlDetails.dwValue * MAX_WAVE_VOLUME /
		(mixerCtrl.Bounds.dwMaximum - mixerCtrl.Bounds.dwMinimum));
}

void
Win32AudioDevice::setOutputVol(int vol)
{
    // AUDIT register function
    if(waveOutHandle == NULL)
    {
	// AUDIT Tried to set volume without opening device.
	return;
    }

    DWORD volume = vol2Dev(vol);
    int err = waveOutSetVolume(waveOutHandle,volume);

    if(err)
    {
	waveOutGetErrorText(err, errorText, sizeof(errorText));
	// AUDIT Error during waveInStart "errorText".
    }
}

int
Win32AudioDevice::getOutputVol()
{
    // AUDIT register function

    if(waveOutHandle == NULL)
    {
	// AUDIT Tried to set volume without opening device.
	return NULL;
    }

    DWORD vol = NULL;

    int err = waveOutGetVolume(waveOutHandle,&vol);

    if(err)
    {
	waveOutGetErrorText(err, errorText, sizeof(errorText));
	// AUDIT Error during waveInStart "errorText".
	return NULL;
    }

    return dev2Vol(vol);
}
