#ifndef WIN32AUDIODEVICE_H_INCLUDED_C56F8B5F
#define WIN32AUDIODEVICE_H_INCLUDED_C56F8B5F

#include <windows.h>
#include <mmsystem.h>
#include <mmreg.h>
#include <AudioDevices/AudioDevice.h>
#include <AudioDevices/Win32DeviceCaps.h>

using namespace std;

//##ModelId=3A902DB700A7

// This can be used to loop through the sampleRates.
static AudioFormat::SampleRate sampleRates[NUM_SAMPLE_RATES] = {
    AudioFormat::S8K,
    AudioFormat::S11K,
    AudioFormat::S16K,
    AudioFormat::S22K,
    AudioFormat::S24K,
    AudioFormat::S32K,
    AudioFormat::S44K,
    AudioFormat::S48K
};

// This can be used to loop through the encodings.
static AudioFormat::Encoding encodings[NUM_ENCODINGS] = {
    AudioFormat::PCM,
    AudioFormat::PCMU,
    AudioFormat::PCMA,
    AudioFormat::S8,
    AudioFormat::U8,
    AudioFormat::S16
};

class Win32AudioDevice : public AudioDevice
{   
    public:

    static const int MAX_WAVE_VOLUME;

    // Since only the lower level device can correctly fill out a device
    // capability class, this function is not inherited from AudioDevice. 
    // The caller of this function is expected to use the DeviceCaps class to
    // find the capabilities of the low level device and then choose how to
    // initialize that device.
    static DeviceCaps* getDevCaps();

    // A DevCap class should describe how the user wants the device to be
    // intialized. The user should call getDevCaps, if they want to know what
    // the device supports.
    Win32AudioDevice();
    ~Win32AudioDevice();

    int open(DevHandle inHandle, DevHandle outHandle);
    bool isOpen();
    void close();

    // The application buffer does not have to be the same size as the block size used by
    // the device. The return value will be the number of bytes copied into the buffer.
    // buffSize is the size of the buffer in bytes.
    int read(sample *appBuff, int buffSize);
    // buffSize is in bytes and the number of bytes read is returned.
    int write(sample *appBuff, int buffSize);

    void setInputVol(int vol);
    int getInputVol();

    void setOutputVol(int vol);
    int getOutputVol();

    AudioFormat::Channel getNumChannels();
    AudioFormat::SampleRate getSampleRate();

    protected:

    private:

    const static int NUM_BLOCKS;
    const static int DEFAULT_BLOCK_SIZE;

    static UINT numInWaves;
    static UINT numOutWaves;

    // Functions to get audio device information.
    static bool getWaveInCaps(int audioDevNum, Win32DeviceCaps *devCaps);
    static bool getWaveOutCaps(int audioDevNum, Win32DeviceCaps *devCaps);
    static void setInEncodings(int waveIn, AudioFormats *audioFormats,
                               AudioFormat::SampleRate rate, 
                               AudioFormat::Channel channels, 
                               AudioFormat::NumBits bits);
    static void setOutEncodings(int waveOut, AudioFormats *audioFormats,
                                AudioFormat::SampleRate rate, 
                                AudioFormat::Channel channels,                             
                                AudioFormat::NumBits bits);
    static int NAL2WaveEncoding(AudioFormat::Encoding encoding);
    static bool NAL2WaveFormat(DWORD dwFormat, AudioFormat::SampleRate rate, 
                               AudioFormat::Channel channel, 
                               AudioFormat::NumBits bits);

    static Win32DeviceCaps *win32DeviceCaps;

    // For error messages from win32.
    char errorText[MAXERRORLENGTH];

    // It may be necessary to do some convertion of volumes so the lower level
    // device understands the 0-100 volume scale of the NAL.
    int vol2Dev(int vol);
    int dev2Vol(int vol);

    WAVEFORMATEX *waveTex;

    DevHandle openInHandle, openOutHandle;
    int readBlockSize, writeBlockSize; 
    bool deviceOpen;
    
    bool waveInClosed;
    HWAVEIN waveInHandle;
    WAVEHDR *waveInHdrs;
    HANDLE readyHandle;
    WAVEHDR *currWaveHdr;
    char *waveInBuf; // buffer used for reading from wave device (microphone)
    int openWaveIn();
    int closeWaveIn();

    bool waveOutClosed;
    HWAVEOUT waveOutHandle;
    WAVEHDR *waveOutHdrs;
    char *waveOutBuf; // buffer used for write to wave device (speakers)
    WAVEHDR* getOutHdr(); 
    int openWaveOut();
    int closeWaveOut();

    HANDLE readThread;
    DWORD readThreadID;
    // This static function knows how to call the function on a class
    // instance.
    static void readProc(Win32AudioDevice *win32Dev, HWAVEIN hwaveIn,
			 UINT uMsg, DWORD dwInstance, DWORD dwParam1,
			 DWORD dwParam2);
    // This function is the function called on the instance.
    void waveInProc(HWAVEIN hwaveIn, UINT uMsg, DWORD dwInstance,
		    DWORD dwParam1, DWORD dwParam2);

    // mixers
    bool mixerClosed;
    HMIXER mixerHandle;
    int openMixer();
    int closeMixer();
};

#endif /* WIN32AUDIODEVICE_H_INCLUDED_C56F8B5F */