// dvi.cpp: implementation of the DVI class.
//
//////////////////////////////////////////////////////////////////////

#include "Codec.h"
#include "cx_dvi.h"
#include "DVIState.h"

#include <iostream>

#ifdef WIN32
#include <winsock2.h>
#else
#include <socket.h>
#endif

using namespace std;

class DVI: public Codec
{
public: 
	virtual void decode(unsigned char *inbuf, short *outbuf, int &length);
	virtual void encode(short *inbuf, unsigned char *outbuf, int &length);

	virtual const CodecFormatList &getSupportedFormats(){
		return formatlist;
	}

	DVI();

private:
	virtual void destroy(){
		delete this;
	}

	CodecFormatList formatlist;
};

CODEC_FACTORY_ENTRY(DVI, dvi)
CODEC_FACTORY_ENTRY(DVI, idvi)

DVI::DVI()
{

	// 20 ms
	formatlist.insert(formatlist.end(),
		CodecFormat(
			"DVI-8K-Mono",
			"IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
			5,4,80,
			AudioFormat(AudioFormat::SIGNED16,8000,16,1,
				160 * AudioFormat::BYTES_PER_SAMPLE
			)
		)
	);

	// 14 ms
	formatlist.insert(formatlist.end(),
		CodecFormat(
			"DVI-11K-Mono",
			"IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
			16,4,80,
			AudioFormat(AudioFormat::SIGNED16,11025,16,1,
				160 * AudioFormat::BYTES_PER_SAMPLE
			)
		)
	);

	// 10 ms
	formatlist.insert(formatlist.end(),
		CodecFormat(
			"DVI-16K-Mono",
			"IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
			6,4,80,
			AudioFormat(AudioFormat::SIGNED16,16000,16,1,
				160 * AudioFormat::BYTES_PER_SAMPLE
			)
		)
	);

	// 7 ms
	formatlist.insert(formatlist.end(),
		CodecFormat(
			"DVI-22K-Mono",
			"IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
			17,4,80,
			AudioFormat(AudioFormat::SIGNED16,22050,16,1,
				160 * AudioFormat::BYTES_PER_SAMPLE
			)
		)
	);

	// 5 ms
	formatlist.insert(formatlist.end(),
		CodecFormat(
			"DVI-32K-Mono",
			"IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
			81,4,80,
			AudioFormat(AudioFormat::SIGNED16,32000,16,1,
				160 * AudioFormat::BYTES_PER_SAMPLE
			)
		)
	);

	// 3.3 ms
	formatlist.insert(formatlist.end(),
		CodecFormat(
			"DVI-48K-Mono",
			"IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
			82,4,80,
			AudioFormat(AudioFormat::SIGNED16,48000,16,1,
				160 * AudioFormat::BYTES_PER_SAMPLE
			)
		)
	);

	

}

void DVI::decode(unsigned char *inbuf, short *outbuf, int &length)
{

	adpcm_state *state = (adpcm_state*)codecState->getState();
	
	state->valprev = ntohs(state->valprev);
	
	length = codecFormat->getAudioFormat().getBytesPerBlock() / sizeof(short);
	adpcm_decoder(inbuf, outbuf, length, state);

}

void DVI::encode(short *inbuf, unsigned char *outbuf, int &length)
{

	int samples;

	adpcm_state *state = (adpcm_state*)codecState->getState();
	
	/* Fix coded state for byte ordering */
	state->valprev = htons(state->valprev);
	
	samples = codecFormat->getFrameSize() * 8 / codecFormat->getAudioFormat().getBitsPerSample();
	outbuf = new unsigned char[samples/sizeof(short)];/* 4 bits per sample */
	length = samples / sizeof(short);

	adpcm_coder(inbuf, outbuf, samples, state);
}