#ifndef AUDIOFORMAT_H_INCLUDED
#define AUDIOFORMAT_H_INCLUDED

static const int NUM_SAMPLE_RATES = 8;
static const int NUM_ENCODINGS = 6;
static const int NUM_CHANNELS = 2;
static const int NUM_BITS = 2;
static const int NUM_BYTES_PER_SAMPLE = 4;

class AudioFormat
{
    public:

    const static int NO_FORMAT;

    typedef enum {
	S8K = 8000,
	S11K = 11025,
	S16K = 16000,
	S22K = 22050,
	S24K = 24000,
	S32K = 32000,
	S44K = 44100,
	S48K = 48000
    } SampleRate;

    // The various encoding types which may be supported by the device.
    typedef enum {
	PCM,
	PCMU, // mu-law (8 bits)
	PCMA, // a-law (8 bits)
	S8,   // signed 8 bits
	U8,   // unsigned 8 bits
	S16   // signed 16 bits
    } Encoding;

    typedef enum {
	MONO = 1,
	STEREO = 2
    } Channel;

    typedef enum {
	B8 = 8,
	B16 = 16
    } NumBits;

 
    AudioFormat()
    {
        sampleRate = (SampleRate)NULL;

        for(int i = 0;i < NUM_ENCODINGS;i++)
	    {
	        encodings[i] = (Encoding)NULL;
	    }
    
	    channels[0] = (Channel)NULL;
        channels[1] = (Channel)NULL;

        bits[0] = (NumBits)NULL;
        bits[1] = (NumBits)NULL;
    };

    AudioFormat(Encoding encoding, SampleRate sampleRate,
		int samplebits, int channels,int blocksize)
		:encoding(encoding),sampleRate(sampleRate),
		bitsPerSample(samplebits),
		numChannels(channels),
		bytesPerBlock(blocksize) {};

    ~AudioFormat () {};

    const inline Encoding getEncoding();
    inline void setEncoding(Encoding encoding);

    const inline int getBitsPerSample();
    inline void setBitsPerSample(int samplebits);

    const inline SampleRate getSampleRate();
    inline void setSampleRate(SampleRate samplerate);

    const inline int getNumChannels();
    inline void setNumChannels(int channels);

    const inline int getBytesPerBlock();
    inline void setBytesPerBlock(int blocksize);

    SampleRate sampleRate;
    Encoding encodings[NUM_ENCODINGS];
    Channel channels[NUM_CHANNELS];
    NumBits bits[NUM_BITS];

    protected:

    Encoding encoding;
    int bitsPerSample;
    int numChannels;
    int bytesPerBlock;

    private:
};

const AudioFormat::Encoding AudioFormat::getEncoding()
{
	return encoding;
}

void AudioFormat::setEncoding(AudioFormat::Encoding encoding)
{
	this->encoding = encoding;
}

const AudioFormat::SampleRate AudioFormat::getSampleRate()
{
	return sampleRate;
}

void AudioFormat::setSampleRate(AudioFormat::SampleRate samplerate)
{
	this->sampleRate = samplerate;
}

const int AudioFormat::getBitsPerSample()
{
	return bitsPerSample;
}

void AudioFormat::setBitsPerSample(int samplebits)
{
	this->bitsPerSample = samplebits;
}

const int AudioFormat::getNumChannels()
{
	return numChannels;
}

void AudioFormat::setNumChannels(int channels)
{
	this->numChannels = channels;
}

const int AudioFormat::getBytesPerBlock()
{
	return bytesPerBlock;
}

void AudioFormat::setBytesPerBlock(int blocksize)
{
	this->bytesPerBlock = blocksize;
}

const int AudioFormat::NO_FORMAT = -1;

#endif
