#ifdef WIN32
#pragma warning (disable:4786)
#endif

#include <Common/AudioFormats.h>

const int NO_FORMAT = -1;
int
AudioFormats::getSize()
{
    return audioFormats.size();
}

void
AudioFormats::checkHandle(FormatHandle formatHandle)
{
    it = audioFormats.find(formatHandle);
    if(it != audioFormats.end())
    {
	AudioFormat audioFormat;
	audioFormats[formatHandle] = audioFormat;
    }
}

void
AudioFormats::setEncoding(FormatHandle formatHandle, AudioFormat::Encoding encoding)
{
    checkHandle(formatHandle);

    switch(encoding)
    {
    case AudioFormat::PCMU:
	    it->second.encodings[0] = encoding;
	case AudioFormat::PCMA:
	    it->second.encodings[1] = encoding;
	case AudioFormat::S8:
	    it->second.encodings[2] = encoding;
	case AudioFormat::U8:
	    it->second.encodings[3] = encoding;
	case AudioFormat::S16:
	    it->second.encodings[4] = encoding;
    };
}

bool
AudioFormats::getEncoding(FormatHandle formatHandle, AudioFormat::Encoding encoding)
{
    for(int i = 0;i < NUM_ENCODINGS;i++)
    {
	if(it->second.encodings[i] == encoding)
	{
	    return true;
	}
    }

    return false;
}

void
AudioFormats::setSampleRate(FormatHandle formatHandle, AudioFormat::SampleRate sampleRate)
{
    checkHandle(formatHandle);
    it->second.sampleRate = sampleRate;
}

bool
AudioFormats::getSampleRate(FormatHandle formatHandle)
{
    it = audioFormats.find(formatHandle);
    if(it != audioFormats.end())
    {
	return true; // a sample rate has been set for this format
    }

    return false;
}

void
AudioFormats::setChannel(FormatHandle formatHandle, AudioFormat::Channel channel)
{
    checkHandle(formatHandle);

    switch(channel)
    {
    case AudioFormat::MONO:
	    it->second.channels[0] = channel;
    case AudioFormat::STEREO:
	    it->second.channels[1] = channel;
    };
}

bool
AudioFormats::getChannel(FormatHandle formatHandle, AudioFormat::Channel channel)
{
    it = audioFormats.find(formatHandle);

    switch(channel)
    {
    case AudioFormat::MONO:
        if(it->second.channels[0] == AudioFormat::MONO)
	    {
		return true;
	    }
    case AudioFormat::STEREO:
        if(it->second.channels[1] == AudioFormat::STEREO)
	    {
		return true;
	    }
	default:
	    return false;
    }
}

bool
AudioFormats::setSampleRateToUse(AudioFormat::SampleRate sampleRate)
{
    for(it = audioFormats.begin();it != audioFormats.end();it++)
    {
	if(it->second.sampleRate == sampleRate)
	{
	    settingsToUse.sampleRate = sampleRate; 
	    return true;
	}
    }

    return false;
}

AudioFormat::SampleRate
AudioFormats::getSampleRateToUse()
{
    return settingsToUse.sampleRate;
}

bool
AudioFormats::setEncodingToUse(AudioFormat::Encoding encoding)
{
    bool found = false;
    for(it = audioFormats.begin();it != audioFormats.end();it++)
    {
	if(it->second.sampleRate == settingsToUse.sampleRate)
	{
	    for(int i = 0;i < NUM_ENCODINGS;i++)
	    {
		if(settingsToUse.encodings[i] == encoding)
		{
		    settingsToUse.encodings[i] = encoding;
		    encodingToUse = i;
		    found = true;
		}
		else
		{
            settingsToUse.encodings[i] = (AudioFormat::Encoding)NULL;
		}
	    }
	}
    }

    return found;
}

AudioFormat::Encoding
AudioFormats::getEncodingToUse()
{
    return settingsToUse.encodings[encodingToUse];
}

bool
AudioFormats::setChannelToUse(AudioFormat::Channel channel)
{
    bool found = false;
    for(it = audioFormats.begin();it != audioFormats.end();it++)
    {
	if(it->second.sampleRate == settingsToUse.sampleRate)
	{
	    for(int i = 0;i < NUM_CHANNELS;i++)
	    {
		if(settingsToUse.channels[i] == channel)
		{
		    settingsToUse.channels[i] = channel;
		    found = true;
		    channelToUse = i;
		}
		else
		{
            settingsToUse.channels[i] = (AudioFormat::Channel)NULL;
		}
	    }
	}
    }

    return found;
}

AudioFormat::Channel
AudioFormats::getChannelToUse()
{
    return settingsToUse.channels[channelToUse];
}
