#ifndef AUDIOFORMATS_H_INCLUDED
#define AUDIOFORMATS_H_INCLUDED

#include <map>
#include <Common/AudioFormat.h>

using namespace std;

typedef int FormatHandle;

// The audio format will allow one to figure out what is supported by a
// device/codec. The question of what can be supported can be asked for every
// sample rate. So a question like:
// "Does sample rate 11025khz support 16Bit PCMU with STEREO?"
// can be asked.
class AudioFormats
{
    public:

    AudioFormats() {}
    ~AudioFormats() {}
    int getSize();

    // Each sample rate can be MONO or STEREO and can support several different
    // encondigs.
    void setSampleRate(FormatHandle formatHandle, AudioFormat::SampleRate sampleRate);
    bool getSampleRate(FormatHandle formatHandle);

    // More than one encoding can be supported per sample rate.
    void setEncoding(FormatHandle formatHandle, AudioFormat::Encoding encoding);
    bool getEncoding(FormatHandle formatHandle, AudioFormat::Encoding encoding);

    // More than one channel can be supported per sample rate.
    void setChannel(FormatHandle formatHandle, AudioFormat::Channel channel);
    bool getChannel(FormatHandle formatHandle, AudioFormat::Channel channel);
    
    protected:

    // Use these functions to indicate to the device which encoding should be
    // used for this run of the session (this time the device is opened). The
    // set functions return false if what is asked for is not supported.

    // The sample rate **HAS TO BE** set first. 
    bool setSampleRateToUse(AudioFormat::SampleRate sampleRate);
    AudioFormat::SampleRate getSampleRateToUse();

    bool setEncodingToUse(AudioFormat::Encoding encoding);
    AudioFormat::Encoding getEncodingToUse();

    bool setChannelToUse(AudioFormat::Channel channel);
    AudioFormat::Channel getChannelToUse();

    private:

    typedef map<int,AudioFormat> AudioFormatMap;

    AudioFormatMap audioFormats;
    AudioFormatMap::iterator it;
    AudioFormat settingsToUse;
    void checkHandle(FormatHandle formatHandle);
    int encodingToUse;
    int channelToUse;
};

#endif
