// RTP.cpp: implementation of the RTP class.
//
//////////////////////////////////////////////////////////////////////

#ifdef WIN32
#pragma warning (disable:4786)
#endif

#include "RTP.h"
#include "RTPException.h"
#include "RTPEvent.h"
#include "RTPSession.h"

double tv2dbl(timeval tv) {return tv.tv_sec + tv.tv_usec / 1000000.0;}

void RTPSchedule(context cid, rtp_opaque_t opaque, timeval *tp);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RTP::RTP()
{

}

RTP::~RTP()
{

}

int RTP::create(RTPSession *session)
{

	session = new RTPSession();

	sessions[session->getContext()] = session;

	return RTP_OK;
}

int RTP::destroy(RTPSession *session)
{
	sessions.erase(session->getContext());

	delete session;

	return RTP_OK;
}

void RTP::schedule(context cid, rtp_opaque_t opaque, timeval *tp)
{
	RTPEvent event(cid,opaque,tp);

	eventQueue.push(event);
}

void RTPSchedule(context cid, rtp_opaque_t opaque, timeval *tp)
{
	RTP::schedule(cid,opaque,tp);
}

void RTP::execute()
{

}

void RTP::_processEvents()
{
	timeval now_tv;
	double now = 0.0;
	RTPEvent event(0,NULL,NULL);
	int rc;

	while(true){

		//how often to check for new events?
		//should I add internal loop that continues
		//	while eventQueue.size > 0 and event.time < now?

		//get current time
		gettimeofday(&now_tv,NULL);
		now = tv2dbl(now_tv);

		//acquire mutex, need exception safe mutex class!!

		//check size of queue
		if(eventQueue.size() > 0){
			//process event?
			event = eventQueue.front();
			
			if(now <= event.getTime()){
				eventQueue.pop();

				rc = ::RTPExecute(event.getContextID(),event.getOpaque());

				if(rc != RTP_OK){
					continue;
				}
			}
			
		}

		//release mutex

	}
}
