// RTP.h: interface for the RTP class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RTP_H__AC38E7D0_5824_46D3_83C2_AC15B5AC7677__INCLUDED_)
#define AFX_RTP_H__AC38E7D0_5824_46D3_83C2_AC15B5AC7677__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <queue>
#include <map>

using namespace std;

#include "RTPCommon.h"

#include "RTPEvent.h"

class RTPSession;

// TODO:  
//	Add mutex to protect event queue
//	Add initialization of event thread

class RTP  
{
public:
	static void execute();
	// Function that is called by RTP library to schedule new events.
	// See RTP documentation for description of parameters.
	static void schedule(context cid, rtp_opaque_t opaque, struct timeval *tp);
	// Destroys an existing RTPSession, throws RTPException on error
	// @param session The session to destroy.
	// @return RTP_OK on success
	static int destroy(RTPSession *session);
	// Creates a new RTPSession, throws RTPException on error.
	// @param session New session pointer is returned through
	// this parameter.
	// @return RTP_OK on success
	static int create(RTPSession *session);
	RTP();
	virtual ~RTP();

protected:
	// Thread function that processes RTP Events.  Events are read
	// from the eventQueue.
	static void _processEvents();
	static double currentTime;
	// The currently scheduled RTP Events.  Events are scheduled by the RTP
	// library calling the schedule function.  Events are processed in the 
	// _processEvents thread function.  The event queue is protected by
	// a mutex
	static queue<RTPEvent> eventQueue;
	// Table of the active RTP Sessions.  Sessions are added by calling
	// create, and removed by calling destroy.
	static map<context,RTPSession *> sessions;
};

#endif // !defined(AFX_RTP_H__AC38E7D0_5824_46D3_83C2_AC15B5AC7677__INCLUDED_)
