// RTPSession.cpp: implementation of the RTPSession class.
//
//////////////////////////////////////////////////////////////////////

#ifdef WIN32
#pragma warning (disable:4786)
#endif

#include "RTPSession.h"
#include "RTPException.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RTPSession::RTPSession()
{
	int rc;	//holder for return code

	rc = ::RTPCreate(&cid);

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::RTPSession: error on call to RTPCreate");
	
	rc = ::RTPSessionAddSendAddr(cid, remote, port, ttl);

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::RTPSession: error on call to RTPSessionAddSendAddr");

	rc = ::RTPMemberInfoSetSDES(cid, 0, RTP_MI_CNAME, cname);

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::RTPSession: error on call to RTPMemberInfoSetSDES");

	rc = ::RTPMemberInfoSetSDES(cid, 0, RTP_MI_NAME, "RTP Example Server");

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::RTPSession: error on call to RTPMemberInfoSetSDES");

	rc = ::RTPOpenConnection(cid);

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::RTPSession: error on call to RTPOpenConnection");
}

RTPSession::~RTPSession()
{
	int rc;	//holder for return code
	rc = ::RTPCloseConnection(cid,"Goodbye!");
	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::~RTPSession: error on call to RTPCloseConnection");

	rc = ::RTPDestroy(cid);
	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::~RTPSession: error on call to RTPDestroy");
}

int RTPSession::send(int tsinc, char marker, short pti, char *payload, int len)
{
	int rc; //holder for return code
	rc = ::RTPSend(cid,tsinc,marker,pti,payload,len);

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::send: error on call to RTPSend");

	return RTP_OK;
}

int RTPSession::getRTPSocket(int *socket)
{
	int rc; //holder for return code
	rc = ::RTPSessionGetRTPSocket(cid,socket);

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::getRTPSocket: error on call to RTPSessionGetRTPSocket");

	return RTP_OK;
}

int RTPSession::setBandwidth(float bandwidth, float rtcpFraction)
{
	int rc; //holder for return code
	rc = ::RTPSessionSetBandwidth(cid, bandwidth, rtcpFraction);

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::setBandwidth: error on call to RTPSessionSetBandwidth");

	return RTP_OK;
}

int RTPSession::execute(RTPEvent &event)
{

}

int RTPSession::getRTCPSocket(int *socket)
{
	int rc; //holder for return code

	rc = ::RTPSessionGetRTCPSocket(cid,socket);

	if(rc != RTP_OK)
		throw RTPException(rc,"RTPSession::getRTCPSocket: error on call to RTPSessionGetRTCPSocket");

	return RTP_OK;
}

void RTPSession::_receive()
{
	//thread function to receive packets from rtp

}
